/* This program blinks a led without the use of the delay function
 * Multitasking framework used is from paper by Loe Feijs entitled Multi-tasking
 * and Arduino: Why and How?  Published in Chen, L.-L., T. Djajadiningrat,
 * L. Feijs, S. Fraser, J. Hu, S. Kyffin and D. Steffen, Eds. (2013).
 * Design and semantics of form and movement. 8th International Conference
 *  on   Design and Semantics of Form and Movement (DeSForM 2013).
 * ISBN 978-90-386-34623,  Wuxi, Philips
 *
 * Software coding Jan Rouvroye for E-lab website.
 * Last edit: September 9, 2015
 */

// using preprocessor commands for settings
#define LED_PIN 13                 // pin where led is connected 
#define LED_OFF 0
#define LED_ON 1
#define LOOPDELAY  20              // delay in ms in loop() for running loop at regular intervals
#define LED_OFF_TIME 20            // number of intervals of LOOPDELAY ms when the led is off 
#define LED_ON_TIME 5              // number of intervals of LOOPDELAY ms when the led is on 

// declarations
int ledState; // variable indicating the state of the led, either LED_OFF or LED_ON
int ledTimer; // counter for the number of intervals of 10 ms when timing is needed

void setup() {
  pinMode(LED_PIN, OUTPUT);
  ledState = LED_OFF;              // initial state is LED_OFF

  ledTimer = LED_OFF_TIME;         // initialize timer counter for LED_OFF state
}

void loop() {
  ledStep();

  delay(LOOPDELAY); // loop once every LOOPINTERVAL ms (if no other processes delay execution)
}

void ledStep() {
  switch (ledState) {
    case LED_ON:
      digitalWrite(LED_PIN, HIGH);
      ledTimer = ledTimer - 1;
      if (ledTimer <= 0)           // change to LED_OFF state
      {
        ledState = LED_OFF;
        ledTimer = LED_OFF_TIME;   // initialize timer counter for LED_OFF state
      }
      break;

    case LED_OFF:
      digitalWrite(LED_PIN, LOW);
      ledTimer = ledTimer - 1;
      if (ledTimer <= 0)           // change to LED_ON state
      {
        ledState = LED_ON;
        ledTimer = LED_ON_TIME;    // initialize timer counter vfor LED_ON state
      }
      break;
  }
}

